@@ -1,7 +1,31 @@
 // vim: tw=80
-use super::*;
+use proc_macro2::{Span, TokenStream};
+use quote::{ToTokens, format_ident, quote};
+use syn::{
+    *,
+    punctuated::Punctuated,
+    spanned::Spanned
+};
 
-use quote::ToTokens;
+
+use crate::{
+    AttrFormatter,
+    HashSet,
+    compile_error,
+    concretize_args,
+    declosurefy,
+    expectation_visibility,
+    gen_keyid,
+    is_concretize,
+    lifetimes_to_generic_params,
+    lifetimes_to_generics,
+    merge_generics,
+    pat_is_self,
+    split_lifetimes,
+    staticize,
+    supersuperfy,
+    supersuperfy_generics,
+};
 
 /// Convert a trait object reference into a reference to a Boxed trait
 ///
@@ -180,11 +204,13 @@ impl<'a> Builder<'a> {
         let mut predty = Vec::new();
         let mut refpredty = Vec::new();
 
-        let (mut declosured_generics, declosured_inputs, call_exprs) =
+        let (mut declosured_generics, declosured_inputs, call_exprs, sig) =
             if self.concretize {
-                concretize_args(&self.sig.generics, &self.sig.inputs)
+                let (x, y, z, sig) = concretize_args(&self.sig.generics, self.sig);
+                (x, y, z, sig)
             } else {
-                declosurefy(&self.sig.generics, &self.sig.inputs)
+                let (x, y, z) = declosurefy(&self.sig.generics, &self.sig.inputs);
+                (x, y, z, self.sig.clone())
             };
         // TODO: make concretize and declosurefy work for the same function
 
@@ -306,7 +332,7 @@ impl<'a> Builder<'a> {
             refpredty,
             return_ref,
             return_refmut,
-            sig: self.sig.clone(),
+            sig,
             struct_: self.struct_.cloned(),
             struct_generics,
             trait_: self.trait_.cloned(),
@@ -445,7 +471,9 @@ impl MockFunction {
     // Supplying modname is an unfortunately hack.  Ideally MockFunction
     // wouldn't need to know that.
     pub fn call(&self, modname: Option<&Ident>) -> impl ToTokens {
-        let attrs = AttrFormatter::new(&self.attrs).format();
+        let attrs = AttrFormatter::new(&self.attrs)
+            .must_use(true)
+            .format();
         let call_exprs = &self.call_exprs;
         let (_, tg, _) = if self.is_method_generic() || self.is_static() {
             &self.egenerics
@@ -518,7 +546,7 @@ impl MockFunction {
                     use ::mockall::{ViaDebug, ViaNothing};
                     let no_match_msg = #no_match_msg;
                     #deref {
-                        let __mockall_guard = #outer_mod_path::EXPECTATIONS
+                        let __mockall_guard = #outer_mod_path::get_expectations()
                             .lock().unwrap();
                         /*
                          * TODO: catch panics, then gracefully release the mutex
@@ -558,7 +586,7 @@ impl MockFunction {
             quote!(
                 #(#attrs)*
                 {
-                    let __mockall_timeses = #inner_mod_ident::EXPECTATIONS.lock()
+                    let __mockall_timeses = #inner_mod_ident::get_expectations().lock()
                         .unwrap()
                         .checkpoint()
                         .collect::<Vec<_>>();
@@ -834,7 +862,7 @@ impl MockFunction {
         quote!(
             #(#attrs)*
             #[allow(missing_docs)]
-            #[allow(clippy::too_many_arguments)]
+            #[allow(clippy::too_many_arguments, clippy::indexing_slicing)]
             pub mod #inner_mod_ident {
                 use super::*;
                 use ::mockall::CaseTreeExt;
@@ -1188,14 +1216,14 @@ impl<'a> ToTokens for CommonExpectationsMethods<'a> {
                     &mut self.0[__mockall_l - 1]
                 }
 
-                #v fn new() -> Self {
-                    Self::default()
+                #v const fn new() -> Self {
+                    Self(Vec::new())
                 }
             }
             impl #ig Default for Expectations #tg #wc
             {
                 fn default() -> Self {
-                    Expectations(Vec::new())
+                    Expectations::new()
                 }
             }
         ).to_tokens(tokens);
@@ -1397,12 +1425,14 @@ impl<'a> ToTokens for ConcreteExpectationGuard<'a> {
         let (ei_ig, _, _) = e_generics.split_for_impl();
         let v = &self.f.privmod_vis;
         quote!(
-            ::mockall::lazy_static! {
-                #[doc(hidden)]
-                #v static ref EXPECTATIONS:
+            #[doc(hidden)]
+            #v fn get_expectations() -> &'static ::std::sync::Mutex<Expectations> {
+                static EXPECTATIONS:
                     ::std::sync::Mutex<Expectations #tg> =
                     ::std::sync::Mutex::new(Expectations::new());
+                &EXPECTATIONS
             }
+
             /// Like an [`&Expectation`](struct.Expectation.html) but
             /// protected by a Mutex guard.  Useful for mocking static
             /// methods.  Forwards accesses to an `Expectation` object.
@@ -1466,11 +1496,12 @@ impl<'a> ToTokens for GenericExpectationGuard<'a> {
         let tbf = tg.as_turbofish();
         let v = &self.f.privmod_vis;
         quote!(
-            ::mockall::lazy_static! {
-                #v static ref EXPECTATIONS:
-                    ::std::sync::Mutex<GenericExpectations> =
-                    ::std::sync::Mutex::new(GenericExpectations::new());
+            #[doc(hidden)]
+            #v fn get_expectations() -> &'static ::std::sync::Mutex<GenericExpectations> {
+                static CELL: ::std::sync::OnceLock<::std::sync::Mutex<GenericExpectations>> = ::std::sync::OnceLock::new();
+                CELL.get_or_init(|| ::std::sync::Mutex::new(GenericExpectations::new()))
             }
+
             /// Like an [`&Expectation`](struct.Expectation.html) but
             /// protected by a Mutex guard.  Useful for mocking static
             /// methods.  Forwards accesses to an `Expectation` object.
@@ -1547,6 +1578,14 @@ impl<'a> ToTokens for Context<'a> {
         #[cfg(feature = "nightly_derive")]
         let must_use = quote!();
 
+        #[cfg(not(feature = "nightly_derive"))]
+        let clear_poison = quote!();
+        #[cfg(feature = "nightly_derive")]
+        let clear_poison = quote!(
+            #[allow(clippy::incompatible_msrv)]
+            get_expectations().clear_poison();
+        );
+
         quote!(
             /// Manages the context for expectations of static methods.
             ///
@@ -1571,7 +1610,7 @@ impl<'a> ToTokens for Context<'a> {
                 }
                 #[doc(hidden)]
                 #v fn do_checkpoint() {
-                    let __mockall_timeses = EXPECTATIONS
+                    let __mockall_timeses = get_expectations()
                         .lock()
                         .unwrap()
                         .checkpoint()
@@ -1583,7 +1622,7 @@ impl<'a> ToTokens for Context<'a> {
                 #v fn expect #meth_ig ( &self,) -> ExpectationGuard #e_tg
                     #meth_wc
                 {
-                    ExpectationGuard::new(EXPECTATIONS.lock().unwrap())
+                    ExpectationGuard::new(get_expectations().lock().unwrap())
                 }
             }
             impl #ty_ig Default for Context #ty_tg #ty_wc {
@@ -1594,10 +1633,13 @@ impl<'a> ToTokens for Context<'a> {
             impl #ty_ig Drop for Context #ty_tg #ty_wc {
                 fn drop(&mut self) {
                     if ::std::thread::panicking() {
+                        // Clear poison since we're about to clear the Mutex's
+                        // contents anyway.
+                        #clear_poison
                         // Drain all expectations so other tests can run with a
                         // blank slate.  But ignore errors so we don't
                         // double-panic.
-                        let _ = EXPECTATIONS
+                        let _ = get_expectations()
                             .lock()
                             .map(|mut g| g.checkpoint().collect::<Vec<_>>());
                     } else {
